/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of JMS SMTP Bridge.
 * 
 * JMS SMTP Bridge is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * JMS SMTP Bridge is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with JMS SMTP Bridge. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.jmsmtpbridge.transform;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.rmi.RemoteException;
import java.util.Hashtable;

import javax.jms.Message;
import javax.jms.TextMessage;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * One of the sample implementations of Jms2SmtpTransformerBean Session Bean.
 * 
 * This class has been provided as a sample to show how to implement the transformation 
 * logic and how to transform a JMS TextMessage into a hashtable of name/value pairs 
 * that will be later read by the SmtpSender Bean to send mail(s). The text in the JMS 
 * message is assumed to be in the form of an xml of the type:
 * 		<msg>
 * 			<to>customer1@abc.com,customer2@def.com</to>
 * 			<cc>customer3@abc.com,customer4@adf.com</cc>
 * 			<bcc>customer5@aa.com,customer6@re.com</bcc>
 * 			<subject>test subject</subject>
 * 			<body><![CDATA[test message]]></body>
 * 			<sendashtml>false</sendashtml>
 * 		</msg>
 * 
 * @author The Shoe Shine Boy
 *
 */
@SuppressWarnings("serial")
public class XmlMsgTransformerBean extends Jms2SmtpTransformerBean {
	// The logger
	private Logger m_logger = Logger.getLogger(getClass().getName());

	/**
	 * The transform() method.
	 * 
	 * Here this method simply loads all the TextMessage into a hashtable and 
	 * return that to the calling method.
	 * 
	 * Basically the TextMessage should match with the xml of type:
	 * 		<msg>
	 * 			<to>customer1@abc.com,customer2@def.com</to>
	 * 			<cc>customer3@abc.com,customer4@adf.com</cc>
	 * 			<bcc>customer5@aa.com,customer6@re.com</bcc>
	 * 			<subject>test subject</subject>
	 * 			<body><![CDATA[test message]]></body>
	 * 			<sendashtml>false</sendashtml>
	 * 		</msg>
	 *
	 * For every field in the above xml such as to, cc, etc. the hashtable names are:
	 * MAIL.TO			Required		The comma separated list of recipients in the to field
	 * MAIL.CC			Optional		The comma separated list of recipients in the cc field
	 * MAIL.BCC			Optional		The comma separated list of recipients in the bcc field
	 * MAIL.SUBJECT		Required		The subject line
	 * MAIL.BODY		Required		The mail body
	 * MAIL.SENDASHTML	Optional		The flag (true/false) telling if the mail needs to be 
	 * 									sent as html or plain/text 
	 */
	@Override
	public Hashtable<String, Object> transform(Message msg)
			throws RemoteException {
		m_logger.info("Inside transform().");
		if (msg instanceof TextMessage) {
			try {
				Hashtable<String, Object> hash = new Hashtable<String, Object>();
				
				InputStream is = new ByteArrayInputStream(((TextMessage)msg).getText().getBytes());
				Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(is);
				m_logger.debug("Loading message into hashtable...");
				loadHash(hash, doc.getFirstChild());
				
				m_logger.debug("Returning hash...");
				return hash;
			} catch (Exception e) {
				throw new RemoteException(e.getMessage());
			}
		} else {
			throw new RemoteException("Expected TextMessage but received " + msg.getClass().getName());
		}
	}

	private void loadHash(Hashtable<String, Object> hash, Node n) {
		m_logger.debug("parsing node: " + n.getNodeName() + " ...");
		if (n.getNodeName().equals("to")) {
			m_logger.debug("Found to: " + n.getFirstChild().getNodeValue());
			if (hash.get(Jms2SmtpTransformer.MAIL_TO) == null) {
				hash.put(Jms2SmtpTransformer.MAIL_TO, n.getFirstChild().getNodeValue());
			} else {
				hash.put(Jms2SmtpTransformer.MAIL_TO, hash.get(Jms2SmtpTransformer.MAIL_TO) + 
						"," + n.getFirstChild().getNodeValue());
			}
		} else if (n.getNodeName().equals("cc")) {
			m_logger.debug("Found cc: " + n.getFirstChild().getNodeValue());
			if (hash.get(Jms2SmtpTransformer.MAIL_CC) == null) {
				hash.put(Jms2SmtpTransformer.MAIL_CC, n.getFirstChild().getNodeValue());
			} else {
				hash.put(Jms2SmtpTransformer.MAIL_CC, hash.get(Jms2SmtpTransformer.MAIL_CC) + 
						"," + n.getFirstChild().getNodeValue());
			}
		} else if (n.getNodeName().equals("bcc")) {
			m_logger.debug("Found bcc: " + n.getFirstChild().getNodeValue());
			if (hash.get(Jms2SmtpTransformer.MAIL_BCC) == null) {
				hash.put(Jms2SmtpTransformer.MAIL_BCC, n.getFirstChild().getNodeValue());
			} else {
				hash.put(Jms2SmtpTransformer.MAIL_BCC, hash.get(Jms2SmtpTransformer.MAIL_BCC) + 
						"," + n.getFirstChild().getNodeValue());
			}
		} else if (n.getNodeName().equals("subject")) {
			m_logger.debug("Found subject: " + n.getFirstChild().getNodeValue());
			hash.put(Jms2SmtpTransformer.MAIL_SUBJECT, n.getFirstChild().getNodeValue());
		} else if (n.getNodeName().equals("body")) {
			m_logger.debug("Found body length: " + (n.getFirstChild().getNodeValue().length()));
			hash.put(Jms2SmtpTransformer.MAIL_BODY, n.getFirstChild().getNodeValue());
		} else if (n.getNodeName().equals("sendashtml")) {
			m_logger.debug("Found sendAsHtml flag: " + n.getFirstChild().getNodeValue());
			hash.put(Jms2SmtpTransformer.MAIL_SENDASHTML, Boolean.valueOf(n.getFirstChild().getNodeValue()));
		} else {
			NodeList children = n.getChildNodes();
			for (int i=0; i<children.getLength(); i++) {
				loadHash(hash, children.item(i));
			}
		}
	}
}
